#include frex:shaders/api/player.glsl
#include frex:shaders/api/view.glsl
#include frex:shaders/api/world.glsl
#include ecos:options

out float v_diffuse;
out float v_heldCosIn;
out float v_heldCosOut;

#ifdef SHADOW_MAP_PRESENT
out vec4 v_shadowViewPos;
#endif

#ifdef ANTIALIASING
#include ecos:shaders/lib/taa.glsl
#endif

void frx_pipelineVertex()
{
  if (frx_isGui)
  {
    gl_Position = frx_guiViewProjectionMatrix * frx_vertex;
  }
  else
  {
    frx_vertex += frx_modelToCamera;
    gl_Position = frx_viewProjectionMatrix * frx_vertex;
  }

  if (frx_isHand)
  {
    frx_vertex = frx_inverseViewMatrix * frx_vertex;
    frx_vertexNormal = frx_normalModelMatrix * frx_vertexNormal;

    #if HAND_SIZE < 1.0
    {
      gl_Position.xy *= HAND_SIZE;
      gl_Position.y -= (1.0 - HAND_SIZE) * 0.5;
    }
    #endif
  }
  
  frx_distance = length(frx_vertex.xyz);

  #ifdef SHADOW_MAP_PRESENT
  {
    v_shadowViewPos = frx_shadowViewMatrix * frx_vertex;
  }
  #endif

  if (frx_isGui && !frx_isHand)
  {
    v_diffuse = clamp(abs(dot(frx_vertexNormal, vec3(-0.5, 1.0, 1.0))), 0.0, 1.0) * 0.5 + 0.5;
  } else {
    v_diffuse = frx_worldHasSkylight == 1 ? max(0.0, dot(frx_vertexNormal, frx_skyLightVector)) : 1.0;
  }

	v_heldCosIn = cos(frx_heldLightInnerRadius * 0.5);
	v_heldCosOut = cos(max(frx_heldLightOuterRadius * 0.5, frx_heldLightInnerRadius * 0.5));

  #ifdef ANTIALIASING
  {
    if (frx_isHand || !frx_isGui)
    {
      gl_Position.xy += taaJitter(1.0 / vec2(frx_viewWidth, frx_viewHeight), frx_renderFrames) * gl_Position.w;

      #ifndef SHADOW_MAP_PRESENT
        // Dirty vanilla shadow offset hack
        gl_Position.z += 0.0018;
      #endif
    }
  }
  #endif
}
