#include ecos:shaders/pass/header.glsl
#include ecos:shaders/lib/tonemap.glsl
#include ecos:options
#include ecos:stylize_options

uniform sampler2D composite_result;
uniform sampler2D bloom_result;

in vec2 texcoord;
out vec4 fragColor;

float rand(vec2 n) { 
	return fract(sin(dot(n, vec2(12.9898, 4.1414))) * 43758.5453);
}

void main()
{
  fragColor.rgb = texture(composite_result, texcoord).rgb;

  #ifdef BLOOM
  {
    vec3 emission = textureLod(bloom_result, texcoord, 0).rgb;
    vec3 bloom = max(textureLod(bloom_result, texcoord, 4).rgb, emission);

    //reverse tonemap
    fragColor.rgb = reverse_tonemap(fragColor.rgb);

    fragColor.rgb += bloom;

    // tonemap again lol
    fragColor.rgb = tonemap(fragColor.rgb);
  }
  #endif

  #if WARM_BOOST != 0
  {
    float warm_boost = abs(WARM_BOOST / 5.0);
  
    #if WARM_BOOST > 0
      const vec3 boosted_color = vec3(1.0, 0.5, 0.0);
    #else
      const vec3 boosted_color = vec3(0.0, 0.5, 1.0);
    #endif

    vec3 frag_maximized = fragColor.rgb / max(0.0001, max(fragColor.r, max(fragColor.g, fragColor.b)));
    float frag_boost = (1.0 - distance(frag_maximized, boosted_color)) * 2.0 - 1.0;
    float boost_amount = clamp(frag_boost, -1.0, 1.0) * warm_boost;

    // skew towards small boost
    boost_amount = boost_amount > 0.0 ? boost_amount * 0.455 : boost_amount * abs(boost_amount) + 0.04;

    float gs = dot(fragColor.rgb, vec3(1.0 / 3.0));
    fragColor.rgb = mix(vec3(gs), fragColor.rgb, 1.0 + boost_amount);
  }
  #endif
  
  #if NOISE > 0
  {
    float noise_amount = NOISE / 100.0;
    float dominant_side = max(frxu_size.x, frxu_size.y);
    vec2 aspect = frxu_size / dominant_side;
    
    vec2 sq_coord = floor(texcoord * 320.0 * aspect) / 320.0;
    vec3 noise_color = vec3(
      rand(sq_coord * 10.0),
      rand(sq_coord * 5.0),
      rand(sq_coord * 7.0)
    );

    float luminance_factor = 0.2 + 0.8 * dot(fragColor.rgb, vec3(1.0 / 3.0));
    
    fragColor.rgb = mix(fragColor.rgb, noise_color, noise_amount * luminance_factor);
  }
  #endif
}
