#include ecos:shaders/pass/header.glsl
#include frex:shaders/api/fog.glsl
#include frex:shaders/api/view.glsl
#include frex:shaders/api/world.glsl
#include frex:shaders/lib/math.glsl
#include ecos:stylize_options

uniform sampler2D img_normal;
uniform sampler2D img_light;

in vec2 texcoord;

out vec3 fragColor;

vec3 color, color0, color1;
// float mult0, mult1;

float corner()
{
  // vec3 color0 = texture(img_normal, texcoord + pixel.xz).rgb;
  // vec3 color1 = texture(img_normal, texcoord + pixel.zy).rgb;

  float diff = max(abs(color0.z - color.z) * 100.0, abs(color1.z - color.z) * 100.0);

  vec2 normal  = color.rg * 2.0 - 1.0;
  vec2 normal0 = color0.rg * 2.0 - 1.0;
  vec2 normal1 = color1.rg * 2.0 - 1.0;

  // float dot0 = dot(normal, normal0);
  // outline = abs(dot0) < abs(dot1) ? dot0 : dot1;

  float dot0 = (normal0.x - normal.x);// * mult0;
  float dot1 = (normal1.y - normal.y);// * mult1;
  float outline = abs(dot0) > abs(dot1) ? dot0 : dot1;

  outline *= min(1.0, 1.0 - clamp(diff - color.z, 0.0, 2.0 * color.z) / (2.0 * color.z)); // edge modulation

  // #ifdef CORNER_HIGHLIGHTS
  // modulate = max(modulate, step(0.0, outline));
  // #endif

  outline *= outline < 0.0 ? 0.5 : 1.0; // softer cocave corners

  return outline;
}

void main()
{
  color = texture(img_normal, texcoord).rgb;
  float width = max(color.z, 0.1);

  // if (color.z * 3.0 < 0.0625 * 0.5) {
  //   fragColor = vec3(0.5);
  //   return;
  // }

	vec3 pixel = vec3((2.0 * frxu_size.y / 1080.0) * max(1.0, 0.75 * float(OUTLINE_WIDTH)) / vec2(frxu_size), 0.0);

	pixel.xy *= pow(width, 1.0 / 2.0) * 2.0;
  // viewNormal.z = sqrt(abs(1.0 - viewNormal.r * viewNormal.r - viewNormal.g * viewNormal.g));

  color0 = texture(img_normal, texcoord + pixel.xz).rgb;
  color1 = texture(img_normal, texcoord + pixel.zy).rgb;

  vec3 colorSwap0 = texture(img_normal, texcoord - pixel.xz).rgb;
  vec3 colorSwap1 = texture(img_normal, texcoord - pixel.zy).rgb;
  // mult0 = 1.0;
  // mult1 = 1.0;

  float outline = 0.0;

  // color0 = texture(img_normal, texcoord + abs(pixel.xz)).rgb;
  // color1 = texture(img_normal, texcoord + abs(pixel.zy)).rgb;

  outline = corner();

  if (colorSwap0.z > color0.z) {
    // pixel.x *= -1.0;
    color0 = colorSwap0;
    // mult0 = -1.0;
  }
  
  if (colorSwap1.z > color1.z) {
    // pixel.y *= -1.0;
    color1 = colorSwap1;
    // mult1 = -1.0;
  }

  float diff = max(abs(color0.z - color.z), abs(color1.z - color.z)) * 100.0;

  float edge = -clamp(diff - 3.0 * width, 0.0, 12.0 * width) / (12.0 * width);
  outline = min(outline, edge + step(0.0, edge));

  float maxz = max(color0.z, color1.z);
  outline *= sqrt(maxz);

  vec3 outline_color = vec3(1.0, 1.0, 0.7);

  const float BLACK_OPACITY = 0.99;

  #ifdef CORNER_HIGHLIGHTS
  {
    outline_color *= outline < 0.0 ? vec3(1.0) : vec3(0.8, 0.8, 0.5);
    outline *= outline < 0.0 ? BLACK_OPACITY : texture(img_light, texcoord).g * 0.25;
  }
  #else
  {
    outline = -abs(outline) * BLACK_OPACITY;
  }
  #endif

  fragColor = outline_color * outline + 0.5;
}
